/*
 * aarbology.cpp
 *
 *  Created on: 26. 3. 2014
 *	  Author: Jan Travnicek
 */

#include <tclap/CmdLine.h>
#include <global/GlobalData.h>
#include <measure>
#include <vector>
#include <sax/FromXMLParserHelper.h>

#include <factory/XmlDataFactory.hpp>
#include <exception/CommonException.h>
#include <tree/Tree.h>
#include <container/Container.h>
#include <container/ObjectsSet.h>

#include <arbology/exact/ExactSubtreeMatch.h>
#include <arbology/exact/ExactPatternMatch.h>
#include <arbology/exact/BoyerMooreHorspool.h>
#include <arbology/exact/ReversedBoyerMooreHorspool.h>
#include <arbology/exact/KnuthMorrisPratt.h>
#include <arbology/exact/DeadZoneUsingBadCharacterShiftAndBorderArray.h>
#include <arbology/exact/ExactSubtreeMatchingAutomaton.h>
#include <arbology/exact/ExactPatternMatchingAutomaton.h>
#include <arbology/exact/ExactSubtreeAutomaton.h>
#include <arbology/exact/ExactTreePatternAutomaton.h>
#include <arbology/exact/ExactSubtreeRepeatsNaive.h>
#include <arbology/transform/BeginToEndIndex.h>

int main ( int argc, char * argv[] ) {
	try {
		common::GlobalData::argc = argc;
		common::GlobalData::argv = argv;

		TCLAP::CmdLine cmd ( "Arbology algorithm access binary", ' ', "0.01" );

		std::vector < std::string > allowed;
		allowed.push_back ( "exactSubtreeMatch" );
		allowed.push_back ( "exactPatternMatch" );
		allowed.push_back ( "boyerMooreHorspool" );
		allowed.push_back ( "reversedBoyerMooreHorspool" );
		allowed.push_back ( "knuthMorrisPratt" );
		allowed.push_back ( "deadZoneUsingBadCharacterShiftAndBorderArray" );
		allowed.push_back ( "exactSubtreeMatchingAutomaton" );
		allowed.push_back ( "exactPatternMatchingAutomaton" );
		allowed.push_back ( "exactSubtreeAutomaton" );
		allowed.push_back ( "exactTreePatternAutomaton" );
		allowed.push_back ( "exactSubtreeRepeatsNaive" );
		TCLAP::ValuesConstraint < std::string > allowedVals ( allowed );

		TCLAP::ValueArg < std::string > algorithm ( "a", "algorithm", "Execute algorithm", false, "exactSubtreeMatch", & allowedVals );
		cmd.add ( algorithm );

		TCLAP::MultiArg < std::string > subjectInput ( "s", "subject", "Subject tree from file", false, "file" );
		cmd.add ( subjectInput );

		TCLAP::MultiArg < std::string > patternInput ( "p", "pattern", "Pattern tree from file", false, "file" );
		cmd.add ( patternInput );

		TCLAP::SwitchArg ends ( "e", "ends", "Return occurrences as end indexes when applicable", false );
		cmd.add ( ends );

		TCLAP::SwitchArg measure ( "m", "measure", "Measure times", false );
		cmd.add ( measure );

		TCLAP::SwitchArg verbose ( "v", "verbose", "Be verbose", false );
		cmd.add ( verbose );

		cmd.parse ( argc, argv );

		if(verbose.isSet())
			common::GlobalData::verbose = true;
		if(measure.isSet())
			common::GlobalData::measure = true;

		measurements::start ( "Overal", measurements::Type::OVERALL );
		measurements::start ( "Input read", measurements::Type::AUXILIARY );

		if ( algorithm.getValue ( ) == "exactSubtreeMatch" ) {
			tree::Tree subject = alib::XmlDataFactory::fromTokens < tree::Tree > ( std::move ( sax::FromXMLParserHelper::parseInput(true, subjectInput).front ( ) ) );
			tree::Tree pattern = alib::XmlDataFactory::fromTokens < tree::Tree > ( std::move ( sax::FromXMLParserHelper::parseInput(true, patternInput).front ( ) ) );

			measurements::end ( );
			measurements::start ( "Algorithm", measurements::Type::MAIN );

			std::set < unsigned > res = arbology::exact::ExactSubtreeMatch::match ( subject, pattern );
			if( ends.isSet ( ) )
				res = arbology::transform::BeginToEndIndex::transform(subject, res);

			measurements::end ( );
			measurements::start ( "Output write", measurements::Type::AUXILIARY );

			alib::XmlDataFactory::toStdout ( res );
		} else if ( algorithm.getValue ( ) == "exactPatternMatch" ) {
			tree::Tree subject = alib::XmlDataFactory::fromTokens < tree::Tree > ( std::move ( sax::FromXMLParserHelper::parseInput(true, subjectInput).front ( ) ) );
			tree::Tree pattern = alib::XmlDataFactory::fromTokens < tree::Tree > ( std::move ( sax::FromXMLParserHelper::parseInput(true, patternInput).front ( ) ) );

			measurements::end ( );
			measurements::start ( "Algorithm", measurements::Type::MAIN );

			std::set < unsigned > res = arbology::exact::ExactPatternMatch::match ( subject, pattern );
			if( ends.isSet ( ) )
				res = arbology::transform::BeginToEndIndex::transform(subject, res);

			measurements::end ( );
			measurements::start ( "Output write", measurements::Type::AUXILIARY );

			alib::XmlDataFactory::toStdout ( res );
		} else if ( algorithm.getValue ( ) == "boyerMooreHorspool" ) {
			tree::Tree subject = alib::XmlDataFactory::fromTokens < tree::Tree > ( std::move ( sax::FromXMLParserHelper::parseInput(true, subjectInput).front ( ) ) );
			tree::Tree pattern = alib::XmlDataFactory::fromTokens < tree::Tree > ( std::move ( sax::FromXMLParserHelper::parseInput(true, patternInput).front ( ) ) );

			measurements::end ( );
			measurements::start ( "Algorithm", measurements::Type::MAIN );

			std::set < unsigned > res = arbology::exact::BoyerMooreHorspool::match ( subject, pattern );
			if( ends.isSet ( ) )
				res = arbology::transform::BeginToEndIndex::transform(subject, res);

			measurements::end ( );
			measurements::start ( "Output write", measurements::Type::AUXILIARY );

			alib::XmlDataFactory::toStdout ( res );
		} else if ( algorithm.getValue ( ) == "reversedBoyerMooreHorspool" ) {
			tree::Tree subject = alib::XmlDataFactory::fromTokens < tree::Tree > ( std::move ( sax::FromXMLParserHelper::parseInput(true, subjectInput).front ( ) ) );
			tree::Tree pattern = alib::XmlDataFactory::fromTokens < tree::Tree > ( std::move ( sax::FromXMLParserHelper::parseInput(true, patternInput).front ( ) ) );

			measurements::end ( );
			measurements::start ( "Algorithm", measurements::Type::MAIN );

			std::set < unsigned > res = arbology::exact::ReversedBoyerMooreHorspool::match ( subject, pattern );
			if( ends.isSet ( ) )
				res = arbology::transform::BeginToEndIndex::transform(subject, res);

			measurements::end ( );
			measurements::start ( "Output write", measurements::Type::AUXILIARY );

			alib::XmlDataFactory::toStdout ( res );
		} else if ( algorithm.getValue ( ) == "knuthMorrisPratt" ) {
			tree::Tree subject = alib::XmlDataFactory::fromTokens < tree::Tree > ( std::move ( sax::FromXMLParserHelper::parseInput(true, subjectInput).front ( ) ) );
			tree::Tree pattern = alib::XmlDataFactory::fromTokens < tree::Tree > ( std::move ( sax::FromXMLParserHelper::parseInput(true, patternInput).front ( ) ) );

			measurements::end ( );
			measurements::start ( "Algorithm", measurements::Type::MAIN );

			std::set < unsigned > res = arbology::exact::KnuthMorrisPratt::match ( subject, pattern );
			if( ends.isSet ( ) )
				res = arbology::transform::BeginToEndIndex::transform(subject, res);

			measurements::end ( );
			measurements::start ( "Output write", measurements::Type::AUXILIARY );

			alib::XmlDataFactory::toStdout ( res );
		} else if ( algorithm.getValue ( ) == "deadZoneUsingBadCharacterShiftAndBorderArray" ) {
			tree::Tree subject = alib::XmlDataFactory::fromTokens < tree::Tree > ( std::move ( sax::FromXMLParserHelper::parseInput(true, subjectInput).front ( ) ) );
			tree::Tree pattern = alib::XmlDataFactory::fromTokens < tree::Tree > ( std::move ( sax::FromXMLParserHelper::parseInput(true, patternInput).front ( ) ) );

			measurements::end ( );
			measurements::start ( "Algorithm", measurements::Type::MAIN );

			std::set < unsigned > res = arbology::exact::DeadZoneUsingBadCharacterShiftAndBorderArray::match ( subject, pattern );
			if( ends.isSet ( ) )
				res = arbology::transform::BeginToEndIndex::transform(subject, res);

			measurements::end ( );
			measurements::start ( "Output write", measurements::Type::AUXILIARY );

			alib::XmlDataFactory::toStdout ( res );
		} else if ( algorithm.getValue ( ) == "exactSubtreeMatchingAutomaton" ) {
			tree::Tree pattern = alib::XmlDataFactory::fromTokens < tree::Tree > ( std::move ( sax::FromXMLParserHelper::parseInput(true, patternInput).front ( ) ) );

			measurements::end ( );
			measurements::start ( "Algorithm", measurements::Type::MAIN );

			automaton::Automaton res = arbology::exact::ExactSubtreeMatchingAutomaton::construct ( pattern );

			measurements::end ( );
			measurements::start ( "Output write", measurements::Type::AUXILIARY );

			alib::XmlDataFactory::toStdout ( res );
		} else if ( algorithm.getValue ( ) == "exactPatternMatchingAutomaton" ) {
			tree::Tree pattern = alib::XmlDataFactory::fromTokens < tree::Tree > ( std::move ( sax::FromXMLParserHelper::parseInput(true, patternInput).front ( ) ) );

			measurements::end ( );
			measurements::start ( "Algorithm", measurements::Type::MAIN );

			automaton::Automaton res = arbology::exact::ExactPatternMatchingAutomaton::construct ( pattern );

			measurements::end ( );
			measurements::start ( "Output write", measurements::Type::AUXILIARY );

			alib::XmlDataFactory::toStdout ( res );
		} else if ( algorithm.getValue ( ) == "exactSubtreeAutomaton" ) {
			tree::Tree subject = alib::XmlDataFactory::fromTokens < tree::Tree > ( std::move ( sax::FromXMLParserHelper::parseInput(true, subjectInput).front ( ) ) );

			measurements::end ( );
			measurements::start ( "Algorithm", measurements::Type::MAIN );

			automaton::Automaton res = arbology::exact::ExactSubtreeAutomaton::construct ( subject );

			measurements::end ( );
			measurements::start ( "Output write", measurements::Type::AUXILIARY );

			alib::XmlDataFactory::toStdout ( res );
		} else if ( algorithm.getValue ( ) == "exactTreePatternAutomaton" ) {
			tree::Tree subject = alib::XmlDataFactory::fromTokens < tree::Tree > ( std::move ( sax::FromXMLParserHelper::parseInput(true, subjectInput).front ( ) ) );

			measurements::end ( );
			measurements::start ( "Algorithm", measurements::Type::MAIN );

			automaton::Automaton res = arbology::exact::ExactTreePatternAutomaton::construct ( subject );

			measurements::end ( );
			measurements::start ( "Output write", measurements::Type::AUXILIARY );

			alib::XmlDataFactory::toStdout ( res );
		} else if ( algorithm.getValue ( ) == "exactSubtreeRepeatsNaive" ) {
			tree::Tree subject = alib::XmlDataFactory::fromTokens < tree::Tree > ( std::move ( sax::FromXMLParserHelper::parseInput(true, subjectInput).front ( ) ) );

			measurements::end ( );
			measurements::start ( "Algorithm", measurements::Type::MAIN );

			tree::Tree res = arbology::exact::ExactSubtreeRepeatsNaive::repeats ( subject );

			measurements::end ( );
			measurements::start ( "Output write", measurements::Type::AUXILIARY );

			alib::XmlDataFactory::toStdout ( res );
		} else {
			throw exception::CommonException ( "Invalid algorithm" );
			return 1;
		}

		measurements::end ( );
		measurements::end ( );

		if ( measure.getValue ( ) ) std::cmeasure << measurements::results ( ) << std::endl;

		return 0;
	} catch ( const exception::CommonException & exception ) {
		alib::XmlDataFactory::toStdout ( exception );
		return 1;
	} catch ( const TCLAP::ArgException & exception ) {
		std::cout << exception.error ( ) << std::endl;
		return 2;
	} catch ( const std::exception & exception ) {
		std::cerr << "Exception caught: " << exception.what ( ) << std::endl;
		return 3;
	} catch ( ... ) {
		std::cerr << "Unknown exception caught." << std::endl;
		return 127;
	}
}
