/*
 * BoyerMooreHorspool.cpp
 *
 *  Created on: 5. 11. 2014
 *      Author: Jan Travnicek
 */

#include "BoyerMooreHorspool.h"
#include "BadCharacterShiftTable.h"
#include "SubtreeJumpTable.h"

#include <tree/Tree.h>
#include <tree/ranked/PrefixRankedBarTree.h>
#include <tree/ranked/PrefixRankedBarPattern.h>
#include <alphabet/RankedSymbol.h>

#include <map>

namespace arbology {

namespace exact {

std::set < unsigned > BoyerMooreHorspool::match ( const tree::Tree & subject, const tree::Tree & pattern ) {
	return dispatch ( subject.getData ( ), pattern.getData ( ) );
}

std::set < unsigned > BoyerMooreHorspool::match ( const tree::PrefixRankedBarTree & subject, const tree::PrefixRankedBarTree & pattern ) {
	return match ( subject, tree::PrefixRankedBarPattern ( pattern ) );
}

auto BoyerMooreHorspoolPrefixRankedBarTreePrefixRankedBarTree = BoyerMooreHorspool::RegistratorWrapper < std::set < unsigned >, tree::PrefixRankedBarTree, tree::PrefixRankedBarTree > ( BoyerMooreHorspool::match );

std::set < unsigned > BoyerMooreHorspool::match ( const tree::PrefixRankedBarTree & subject, const tree::PrefixRankedBarPattern & pattern ) {
	std::set < unsigned > occ;
	std::map < alphabet::RankedSymbol, size_t > bcs = BadCharacterShiftTable::bcs ( pattern ); //NOTE: the subjects alphabet must be a subset or equal to the pattern
	std::vector < int > subjectSubtreeJumpTable = SubtreeJumpTable::compute ( subject );

	 // index to the subject
	unsigned i = 0;

	 // main loop of the algorithm over all possible indexes where the pattern can start
	while ( i + pattern.getContent ( ).size ( ) <= subject.getContent ( ).size ( ) ) {

		 // index to the pattern
		unsigned j = pattern.getContent ( ).size ( ) - 1;

		 // offset to the subject
		unsigned offset = i + j;

		while ( ( j > 0 ) && ( offset > 0 ) ) {
			if ( subject.getContent ( )[offset] == pattern.getContent ( )[j] ) {
				 // match of symbol
				offset = offset - 1;
				j = j - 1;
			} else if ( ( pattern.getContent ( )[j] == pattern.getVariablesBar ( ) ) && ( pattern.getBars ( ).count ( subject.getContent ( )[offset] )) ) {
				 // match of variable with subtree
				offset = subjectSubtreeJumpTable[offset];
				j = j - 2;
			} else {
				break;
			}
		}

		 // match was found
		if ( j == 0 ) occ.insert ( offset );

		 // shift heuristics
		i += bcs[subject.getContent ( )[i + pattern.getContent ( ).size ( ) - 1]];
	}

	return occ;
}

auto BoyerMooreHorspoolPrefixRankedBarTreePrefixRankedBarPattern = BoyerMooreHorspool::RegistratorWrapper < std::set < unsigned >, tree::PrefixRankedBarTree, tree::PrefixRankedBarPattern > ( BoyerMooreHorspool::match );

} /* namespace exact */

} /* namespace arbology */
