/*
 * EpsilonRemoverOutgoing.cpp
 *
 *  Created on: 16. 1. 2014
 *	  Author: Tomas Pecka
 */

#include "EpsilonRemoverOutgoing.h"

#include "../properties/EpsilonClosure.h"

namespace automaton {

namespace simplify {

automaton::DFA EpsilonRemoverOutgoing::remove(const automaton::DFA& origFSM) {
	return origFSM;
}

auto EpsilonRemoverOutgoingDFA = EpsilonRemoverOutgoing::RegistratorWrapper<automaton::DFA, automaton::DFA>(EpsilonRemoverOutgoing::remove);

automaton::MultiInitialStateNFA EpsilonRemoverOutgoing::remove(const automaton::MultiInitialStateNFA& origFSM) {
	return origFSM;
}

auto EpsilonRemoverOutgoingMultiInitialStateNFA = EpsilonRemoverOutgoing::RegistratorWrapper<automaton::MultiInitialStateNFA, automaton::MultiInitialStateNFA>(EpsilonRemoverOutgoing::remove);

automaton::NFA EpsilonRemoverOutgoing::remove(const automaton::NFA& origFSM) {
	return origFSM;
}

auto EpsilonRemoverOutgoingNFA = EpsilonRemoverOutgoing::RegistratorWrapper<automaton::NFA, automaton::NFA>(EpsilonRemoverOutgoing::remove);

automaton::MultiInitialStateNFA EpsilonRemoverOutgoing::remove( const automaton::EpsilonNFA & origFSM ) {
	automaton::MultiInitialStateNFA fsm;
	fsm.setStates( origFSM.getStates() );
	fsm.setFinalStates( origFSM.getFinalStates() );
	fsm.setInputAlphabet( origFSM.getInputAlphabet() );

	/**
	 * Step 1 from Melichar 2.41
	 */
	for( const auto & middle : origFSM.getStates( ) ) {
		const std::set<automaton::State> middleClosure = automaton::properties::EpsilonClosure::epsilonClosure( origFSM, middle );
		for( const auto & symbol : origFSM.getInputAlphabet() ) {
			for( const auto& transition : origFSM.getTransitions( ) ) {
				if(transition.second.find(middle) == transition.second.end()) continue;
				if(transition.first.second != symbol) continue;

				for( const auto & to : middleClosure )
					fsm.addTransition(transition.first.first, symbol, to );
			}
		}
	}

	/**
	 * Step 2 from Melichar 2.41
	 */
	const std::set<automaton::State> cl = automaton::properties::EpsilonClosure::epsilonClosure( origFSM, origFSM.getInitialState() );
	fsm.setInitialStates( cl );

	return fsm;
}

auto EpsilonRemoverOutgoingEpsilonNFA = EpsilonRemoverOutgoing::RegistratorWrapper<automaton::MultiInitialStateNFA, automaton::EpsilonNFA>(EpsilonRemoverOutgoing::remove);

automaton::Automaton EpsilonRemoverOutgoing::remove(const automaton::Automaton& automaton) {
	return dispatch(automaton.getData());
}

} /* namespace simplify */

} /* namespace automaton */
