/*
 * Total.cpp
 *
 *  Created on: 20. 9. 2014
 *	  Author: Tomas Pecka
 */

#include "Total.h"

#include <exception/CommonException.h>

#include <automaton/FSM/NFA.h>
#include <automaton/FSM/DFA.h>

namespace automaton {

namespace simplify {

automaton::Automaton Total::total(const Automaton& automaton) {
	return dispatch(automaton.getData());
}

automaton::NFA Total::total(const automaton::NFA& automaton) {
	if(! automaton.isDeterministic()) {
		throw exception::CommonException("Automaton must be deterministic to make its transition function total");
	}

	automaton::NFA res(automaton);
	automaton::State nullState = automaton::createUniqueState(automaton::State("q0"), automaton.getStates());
	res.addState(nullState);

	for(const auto& q : res.getStates()) {
		for(const auto& a : res.getInputAlphabet()) {
			if(res.getTransitions().find(std::make_pair(q, a)) == res.getTransitions().end()) {
				res.addTransition(q, a, nullState);
			}
		}
	}

	return res;
}

auto TotalNFA = Total::RegistratorWrapper<automaton::NFA, automaton::NFA>(Total::total);

automaton::DFA Total::total(const automaton::DFA& automaton) {
	automaton::DFA res(automaton);
	automaton::State nullState = automaton::createUniqueState(automaton::State("q0"), automaton.getStates());
	res.addState(nullState);

	for(const auto& q : res.getStates()) {
		for(const auto& a : res.getInputAlphabet()) {
			if(res.getTransitions().find(std::make_pair(q, a)) == res.getTransitions().end()) {
				res.addTransition(q, a, nullState);
			}
		}
	}

	return res;
}

auto TotalDFA = Total::RegistratorWrapper<automaton::DFA, automaton::DFA>(Total::total);

} /* namespace simplify */

} /* namespace automaton */

