/*
 * Compaction.cpp
 *
 *  Created on: 2. 11. 2014
 *	  Author: Tomas Pecka
 */

#include "Compaction.h"

#include <automaton/common/State.h>
#include <automaton/FSM/CompactNFA.h>
#include <automaton/FSM/DFA.h>

#include <stack>
#include <tuple>

namespace automaton {

namespace transform {

automaton::Automaton Compaction::convert(const automaton::Automaton& automaton) {
	return dispatch(automaton.getData());
}

automaton::CompactNFA Compaction::convert(const automaton::CompactNFA& automaton) {
	return automaton;
}

auto CompactionCompactNFA = Compaction::RegistratorWrapper<automaton::CompactNFA, automaton::CompactNFA>(Compaction::convert);

automaton::CompactNFA Compaction::convert(const automaton::DFA& automaton) {
	automaton::CompactNFA res(automaton.getInitialState());
	res.setInputAlphabet(automaton.getInputAlphabet());

	std::set<automaton::State> visited;

	std::stack<std::tuple<automaton::State, automaton::State, alphabet::Symbol>> stack; // state x lastFork x symbol we used to go to that state
	for(const auto& transition: automaton.getTransitionsFromState(automaton.getInitialState()))
		stack.push(std::make_tuple(transition.second, automaton.getInitialState(), transition.first.second));

	if(automaton.getFinalStates().count(automaton.getInitialState()) > 0)
		res.addFinalState(automaton.getInitialState());

	while(!stack.empty()) {
		automaton::State q = std::move(std::get<0>(stack.top()));
		automaton::State lastFork = std::move(std::get<1>(stack.top()));
		alphabet::Symbol symbol = std::move(std::get<2>(stack.top()));
		stack.pop();

		string::LinearString path { automaton.getInputAlphabet(), { symbol } };

		std::map<std::pair<automaton::State, alphabet::Symbol>, automaton::State> transitions;
		// only 1 child and nonfinal
		while((transitions = automaton.getTransitionsFromState(q)).size() == 1 && automaton.getFinalStates().count(q) == 0) {
			const std::pair<std::pair<automaton::State, alphabet::Symbol>, automaton::State>& transition = * transitions.begin();
			path.appendSymbol(transition.first.second);
			q = transition.second;
		}

		// fork or final state
		res.addState(q);

		if(automaton.getFinalStates().count(q))
			res.addFinalState(q);

		res.addTransition(lastFork, path, q);

		for(const std::pair<const std::pair<automaton::State, alphabet::Symbol>, automaton::State>& transition : automaton.getTransitionsFromState(q))
			if(visited.insert(transition.second).second)
				stack.push(std::make_tuple(transition.second, q, transition.first.second));

	}

	return res;
}

auto CompactionDFA = Compaction::RegistratorWrapper<automaton::CompactNFA, automaton::DFA>(Compaction::convert);

automaton::CompactNFA Compaction::convert(const automaton::NFA& automaton) {
	automaton::CompactNFA res(automaton.getInitialState());
	// TODO
	return res;
}

auto CompactionNFA = Compaction::RegistratorWrapper<automaton::CompactNFA, automaton::NFA>(Compaction::convert);

} /* namespace transform */

} /* namespace automaton */
