/*
 * Reverse.cpp
 *
 *  Created on: 7. 11. 2014
 *	  Author: Tomas Pecka
 */

#include "Reverse.h"

#include <automaton/FSM/MultiInitialStateNFA.h>

#include <vector>
#include <algorithm>

namespace automaton {

namespace transform {

automaton::Automaton Reverse::convert(const Automaton& automaton) {
	return dispatch(automaton.getData());
}

automaton::MultiInitialStateNFA Reverse::convert(const automaton::DFA& automaton) {
	automaton::MultiInitialStateNFA res;

	res.setStates(automaton.getStates());
	res.addFinalState(automaton.getInitialState());
	res.setInitialStates( automaton.getFinalStates() );
	res.setInputAlphabet(automaton.getInputAlphabet());

	for(const auto& t : automaton.getTransitions())
		res.addTransition(t.second, t.first.second, t.first.first);

	return res;
}

auto ReverseDFA = Reverse::RegistratorWrapper<automaton::MultiInitialStateNFA, automaton::DFA>(Reverse::convert);

automaton::MultiInitialStateNFA Reverse::convert(const automaton::NFA& automaton) {
	automaton::MultiInitialStateNFA res;

	res.setStates(automaton.getStates());
	res.addFinalState(automaton.getInitialState());
	res.setInitialStates( automaton.getFinalStates() );
	res.setInputAlphabet(automaton.getInputAlphabet());

	for(const auto& t : automaton.getTransitions())
		for(const auto& q : t.second)
			res.addTransition(q, t.first.second, t.first.first);

	return res;
}

auto ReverseNFA = Reverse::RegistratorWrapper<automaton::MultiInitialStateNFA, automaton::NFA>(Reverse::convert);

automaton::MultiInitialStateNFA Reverse::convert(const automaton::MultiInitialStateNFA& automaton) {
	automaton::MultiInitialStateNFA res;

	res.setStates(automaton.getStates());
	res.setFinalStates(automaton.getInitialStates());
	res.setInitialStates( automaton.getFinalStates() );
	res.setInputAlphabet(automaton.getInputAlphabet());

	for(const auto& t : automaton.getTransitions())
		for(const auto& q : t.second)
			res.addTransition(q, t.first.second, t.first.first);

	return res;
}

auto ReverseMultiInitialStateNFA = Reverse::RegistratorWrapper<automaton::MultiInitialStateNFA, automaton::MultiInitialStateNFA>(Reverse::convert);

} /* namespace transform */

} /* namespace automaton */

