/*
 * RegExpEpsilon.cpp
 *
 *  Created on: 19. 1. 2014
 *	  Author: Tomas Pecka
 */

#include "RegExpEpsilon.h"
#include <regexp/formal/FormalRegExpElements.h>
#include <regexp/unbounded/UnboundedRegExpElements.h>

namespace regexp {

namespace properties {

bool RegExpEpsilon::languageContainsEpsilon(const regexp::RegExp& regexp) {
	return dispatch(regexp.getData());
}

bool RegExpEpsilon::languageContainsEpsilon(const regexp::FormalRegExpElement& regexp) {
	bool out;
	regexp.Accept((void*) &out, RegExpEpsilon::REG_EXP_EPSILON);
	return out;
}

bool RegExpEpsilon::languageContainsEpsilon(const regexp::FormalRegExp& regexp) {
	return languageContainsEpsilon(regexp.getRegExp());
}

auto RegExpEpsilonFormalRegExp = RegExpEpsilon::RegistratorWrapper<bool, regexp::FormalRegExp>( RegExpEpsilon::languageContainsEpsilon);

bool RegExpEpsilon::languageContainsEpsilon(const regexp::UnboundedRegExpElement& regexp) {
	bool out;
	regexp.Accept((void*) &out, RegExpEpsilon::REG_EXP_EPSILON);
	return out;
}

bool RegExpEpsilon::languageContainsEpsilon(const regexp::UnboundedRegExp& regexp) {
	return languageContainsEpsilon(regexp.getRegExp());
}

auto RegExpEpsilonUnboundedRegExp = RegExpEpsilon::RegistratorWrapper<bool, regexp::UnboundedRegExp>( RegExpEpsilon::languageContainsEpsilon);

// ---------------------------------------------------------------------------

void RegExpEpsilon::Visit(void* data, const regexp::UnboundedRegExpAlternation& alternation) const {
	bool &ret = *(bool*) data;

	for(const auto& element : alternation.getElements()) {
		static_cast<const regexp::UnboundedRegExpElement&>(*element).Accept(data, RegExpEpsilon::REG_EXP_EPSILON);
		if(ret) {
			ret = true;
			return;
		}
	}
	
	ret = false;
}

void RegExpEpsilon::Visit(void* data, const regexp::UnboundedRegExpConcatenation& concatenation) const {
	bool &ret = *(bool*) data;

	for(const auto& element : concatenation.getElements()) {
		static_cast<const regexp::UnboundedRegExpElement&>(*element).Accept(data, RegExpEpsilon::REG_EXP_EPSILON);
		if(!ret) {
			ret = false;
			return;
		}
	}
	
	ret = true;
}

void RegExpEpsilon::Visit(void* data, const regexp::UnboundedRegExpIteration&) const {
	bool &ret = *(bool*) data;
	ret = true;
}

void RegExpEpsilon::Visit(void* data, const regexp::UnboundedRegExpSymbol&) const {
	bool &ret = *(bool*) data;
	ret = false;
}

void RegExpEpsilon::Visit(void* data, const regexp::UnboundedRegExpEpsilon&) const {
	bool &ret = *(bool*) data;
	ret = true;
}

void RegExpEpsilon::Visit(void* data, const regexp::UnboundedRegExpEmpty&) const {
	bool &ret = *(bool*) data;
	ret = false;
}

// ----------------------------------------------------------------------------

void RegExpEpsilon::Visit(void* data, const regexp::FormalRegExpAlternation& alternation) const {
	bool &ret = *(bool*) data;

	static_cast<const regexp::FormalRegExpElement&>(alternation.getLeftElement()).Accept(data, RegExpEpsilon::REG_EXP_EPSILON);
	bool retLeft = ret;

	static_cast<const regexp::FormalRegExpElement&>(alternation.getRightElement()).Accept(data, RegExpEpsilon::REG_EXP_EPSILON);
	bool retRight = ret;

	ret = retLeft || retRight;
}

void RegExpEpsilon::Visit(void* data, const regexp::FormalRegExpConcatenation& concatenation) const {
	bool &ret = *(bool*) data;

	static_cast<const regexp::FormalRegExpElement&>(concatenation.getLeftElement()).Accept(data, RegExpEpsilon::REG_EXP_EPSILON);
	bool retLeft = ret;

	static_cast<const regexp::FormalRegExpElement&>(concatenation.getRightElement()).Accept(data, RegExpEpsilon::REG_EXP_EPSILON);
	bool retRight = ret;

	ret = retLeft && retRight;
}

void RegExpEpsilon::Visit(void* data, const regexp::FormalRegExpIteration&) const {
	bool &ret = *(bool*) data;
	ret = true;
}

void RegExpEpsilon::Visit(void* data, const regexp::FormalRegExpSymbol&) const {
	bool &ret = *(bool*) data;
	ret = false;
}

void RegExpEpsilon::Visit(void* data, const regexp::FormalRegExpEmpty&) const {
	bool &ret = *(bool*) data;
	ret = false;
}

void RegExpEpsilon::Visit(void* data, const regexp::FormalRegExpEpsilon&) const {
	bool &ret = *(bool*) data;
	ret = true;
}

// ---------------------------------------------------------------------------

const RegExpEpsilon RegExpEpsilon::REG_EXP_EPSILON;

} /* namespace properties */

} /* namespace regexp */
