/*
 * BoyerMooreHorspool.cpp
 *
 *  Created on: 5. 11. 2014
 *      Author: Radomir Polach, Tomas Pecka
 */

#include "BoyerMooreHorspool.h"
#include "BadCharacterShiftTable.h"

#include <string/LinearString.h>
#include <alphabet/Symbol.h>

#include <map>
#include <measure>

namespace stringology {

namespace exact {

std::set<unsigned> BoyerMooreHorspool::match(const string::String& subject, const string::String& pattern) {
	return dispatch(subject.getData(), pattern.getData());
}

std::set<unsigned> BoyerMooreHorspool::match(const string::LinearString& string, const string::LinearString& pattern)
{
	std::set<unsigned> occ;

	measurements::start ( "Preprocess", measurements::Type::PREPROCESS );
	std::map<alphabet::Symbol, size_t> bcs = BadCharacterShiftTable::bcs(pattern); //NOTE: the subjects alphabet must be a subset or equal to the pattern
	measurements::end ( );

	measurements::start ( "Algorithm", measurements::Type::ALGORITHM );
	size_t haystack_offset = 0;
	while(haystack_offset + pattern.getContent().size() <= string.getContent().size()) {
		size_t i = pattern.getContent().size();
		while(i > 0 && string.getContent()[haystack_offset + i - 1] == pattern.getContent()[i - 1]) {
			i--;
		}

		// Yay, there is match!!!
		if(i == 0) occ.insert(haystack_offset);
		haystack_offset += bcs[string.getContent()[haystack_offset + pattern.getContent().size() - 1]];
		//std::cout << haystack_offset << std::endl;
	}
	measurements::end ( );

	return occ;
}

auto BoyerMooreHorpoolLinearStringLinearString = BoyerMooreHorspool::RegistratorWrapper<std::set<unsigned>, string::LinearString, string::LinearString>(BoyerMooreHorspool::match);

} /* namespace exact */

} /* namespace stringology */
