/*
 * SuffixTrie.cpp
 *
 *  Created on: 1. 11. 2014
 *      Author: Tomas Pecka
 */

#include "SuffixTrie.h"

#include <string/LinearString.h>
#include <string/LinearStringTerminatingSymbol.h>
#include <string/Epsilon.h>
#include <alphabet/EndSymbol.h>

namespace stringology {

namespace indexing {

indexes::SuffixTrieFinalMark SuffixTrie::construct ( const string::String & string ) {
	return dispatch ( string.getData ( ) );
}

indexes::SuffixTrieFinalMark SuffixTrie::construct ( const string::LinearString & w ) {
	indexes::SuffixTrieFinalMark res ( w.getAlphabet ( ) );

	for ( unsigned int i = 0; i < w.getContent ( ).size ( ); i++ ) {
		unsigned int k = i;
		indexes::SuffixTrieNodeFinalMark * n = & res.getRoot ( );

		 // inlined slow_find_one from MI-EVY lectures
		while ( k < w.getContent ( ).size ( ) && n->hasChild ( w.getContent ( )[k] ) )
			n = & n->getChild ( w.getContent ( )[k++] );

		for ( ; k < w.getContent ( ).size ( ); k++ )
			n = & n->addChild ( w.getContent ( )[k], indexes::SuffixTrieNodeFinalMark ( { }, false ) );

		n->setFinalMark ( true );
	}

	return res;
}

auto SuffixTrieLinearString = SuffixTrie::RegistratorWrapper < indexes::SuffixTrieFinalMark, string::LinearString > ( SuffixTrie::construct );

indexes::SuffixTrieTerminatingSymbol SuffixTrie::construct ( const string::LinearStringTerminatingSymbol & w ) {
	indexes::SuffixTrieTerminatingSymbol res ( w.getAlphabet ( ), w.getTerminatingSymbol ( ) );

	for ( unsigned int i = 0; i < w.getContent ( ).size ( ); i++ ) {
		unsigned int k = i;
		indexes::SuffixTrieNodeTerminatingSymbol * n = & res.getRoot ( );

		 // inlined slow_find_one from MI-EVY lectures
		while ( k < w.getContent ( ).size ( ) && n->hasChild ( w.getContent ( )[k] ) )
			n = & n->getChild ( w.getContent ( )[k++] );

		for ( ; k < w.getContent ( ).size ( ); k++ )
			n = & n->addChild ( w.getContent ( )[k], indexes::SuffixTrieNodeTerminatingSymbol ( { } ) );
	}

	return res;
}

auto SuffixTrieTerminatingSymbolLinearStringTerminatingSymbol = SuffixTrie::RegistratorWrapper < indexes::SuffixTrieTerminatingSymbol, string::LinearStringTerminatingSymbol > ( SuffixTrie::construct );

} /* namespace indexing */

} /* namespace stringology */
