#include "RegExpDerivationTest.h"

#include "regexp/transform/RegExpDerivation.h"

#include <factory/StringDataFactory.hpp>

#define CPPUNIT_IMPLY(x, y) CPPUNIT_ASSERT(!(x) || (y))

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( RegExpDerivationTest, "regexp" );
CPPUNIT_TEST_SUITE_REGISTRATION( RegExpDerivationTest );

void RegExpDerivationTest::setUp() {
}

void RegExpDerivationTest::tearDown() {
}

void RegExpDerivationTest::testRegExpDerivation() {
    // https://edux.fit.cvut.cz/oppa/BI-AAG/cviceni/aag-cv04.pdf

    // Exc. 13
    this->ExecSingleTest("0 1 0 + 1 0 1 + 0*1 + 1*0", "0", "#E 1 0+#0 0 1+#E 0* 1+#0+#0 1* 0+#E");
    this->ExecSingleTest("0 1 0 + 1 0 1 + 0*1 + 1*0", "0 0", "#0 1 0+#0 0+#0 0 1+(#0 0*+#E 0*) 1+#0+#0+#0 1* 0+#0");

    // Exc. 14
    this->ExecSingleTest("(0 1*0 + 1 0*)*", "1", "(#0 1* 0+#E 0*) (0 1* 0+1 0*)*");
    this->ExecSingleTest("(0 1*0 + 1 0*)*", "1 0", "(#0 1* 0+#0 0*+#E 0*) (0 1* 0+1 0*)*+(#E 1* 0+#0 0*) (0 1* 0+1 0*)*");

    // Exc. 15
    this->ExecSingleTest("1 0*1*0", "1 0 0", "#0 0* 1* 0+(#0 0*+#E 0*) 1* 0+#0 1* 0+#E+#0 1* 0+#0");
}

void RegExpDerivationTest::ExecSingleTest(std::string regexp_str, std::string string_str, std::string result) {
    regexp::RegExp regexp = alib::StringDataFactory::fromString<regexp::RegExp>(regexp_str);

    string::LinearString string = static_cast<const string::LinearString&>(alib::StringDataFactory::fromString<string::String>("\"" + string_str + "\"").getData());

    std::cout << alib::StringDataFactory::toString<regexp::RegExp>(regexp::RegExpDerivation::derivation(regexp, string)) << " == " << std::endl << result << std::endl << std::endl;

    CPPUNIT_ASSERT(alib::StringDataFactory::toString<regexp::RegExp>(regexp::RegExpDerivation::derivation(regexp, string)) == result);
}
