/*
 * WrapperBase.h
 *
 *  Created on: Apr 10, 2013
 *      Author: Jan Travnicek
 */

#ifndef WRAPPER_BASE_H_
#define WRAPPER_BASE_H_

#include <ostream>
#include <memory>

namespace alib {

class WrapperBaseBase {
};

template < typename T >
class WrapperBase : public WrapperBaseBase {
protected:
	std::cow_shared_ptr < T > data;

private:
	void unify ( WrapperBase & other ) {
		if ( this->data.getUseCount ( ) > other.data.getUseCount ( ) )
			other.data = this->data;
		else
			this->data = other.data;
	}

public:
	explicit WrapperBase ( T * data ) : data ( data ) {
	}

	explicit WrapperBase ( const T & data ) : data ( data.clone ( ) ) {
	}

	explicit WrapperBase ( T && data ) : data ( std::move ( data ).plunder ( ) ) {
	}

	virtual ~WrapperBase ( ) noexcept {
	}

	const T & getData ( ) const {
		return * data;
	}

	T & getData ( ) {
		return * data;
	}

	void setData ( T * data ) {
		this->data = std::cow_shared_ptr < T > ( data );
	}

	void setData ( const T & data ) {
		this->data = std::cow_shared_ptr < T > ( data.clone ( ) );
	}

	void setData ( T && data ) {
		this->data = std::cow_shared_ptr < T > ( std::move ( data ).plunder ( ) );
	}

	bool operator >=( const WrapperBase & other ) const {
		return this->compare ( other ) >= 0;
	}

	bool operator <=( const WrapperBase & other ) const {
		return this->compare ( other ) <= 0;
	}

	bool operator >( const WrapperBase & other ) const {
		return this->compare ( other ) > 0;
	}

	bool operator <( const WrapperBase & other ) const {
		return this->compare ( other ) < 0;
	}

	bool operator !=( const WrapperBase & other ) const {
		return this->compare ( other ) != 0;
	}

	bool operator ==( const WrapperBase & other ) const {
		return this->compare ( other ) == 0;
	}

	int compare ( const WrapperBase & other ) const {
		if ( this->data.get ( ) == other.data.get ( ) ) return 0;

		int res = ( * this->data ).compare ( * other.data );

		if ( res == 0 ) const_cast < WrapperBase * > ( this )->unify ( const_cast < WrapperBase & > ( other ) );

		return res;
	}

	friend std::ostream & operator <<( std::ostream & os, const WrapperBase & instance ) {
		os << * ( instance.data );
		return os;
	}

	explicit operator std::string ( ) const {
		return ( std::string ) * data;
	}
};

} /* namespace alib */

namespace std {

template < class T >
struct compare < T, typename std::enable_if < std::is_base_of < alib::WrapperBaseBase, T >::value >::type > {
	int operator()(const T& first, const T& second) const {
		return first.getData ( ).compare ( second.getData ( ) );
	}
};

} /* namespace std */

#endif /* WRAPPER_BASE_H_ */
