#include "ComponentsTest.h"
#include <core/components.hpp>

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION ( ComponentsTest, "alphabet" );
CPPUNIT_TEST_SUITE_REGISTRATION ( ComponentsTest );

struct GeneralAlphabet {
};

struct NonlinearAlphabet {
};

struct SubtreeWildcard {
};

class A : public std::Components < A, std::string, std::tuple < GeneralAlphabet, NonlinearAlphabet >, std::tuple < SubtreeWildcard > > {
public:
	A ( std::string string ) : std::Components < A, std::string, std::tuple < GeneralAlphabet, NonlinearAlphabet >, std::tuple < SubtreeWildcard > > ( std::make_tuple ( std::set < std::string > { string, "aaa" }, std::set < std::string > { "aaa" } ), std::make_tuple ( string ) ) {
	}
};

namespace std {

template < >
bool A::Component < A, std::string, GeneralAlphabet >::used ( const std::string & string ) const {
	return static_cast < const A * > ( this )->accessComponent < NonlinearAlphabet > ( ).get ( ).count ( string ) || static_cast < const A * > ( this )->accessElement < SubtreeWildcard > ( ).get ( ) == string;
}

template < >
bool A::Component < A, std::string, NonlinearAlphabet >::used ( const std::string & ) const {
	return false;
}

template < >
bool A::Component < A, std::string, GeneralAlphabet >::available ( const std::string & ) const {
	return true;
}

template < >
bool A::Component < A, std::string, NonlinearAlphabet >::available ( const std::string & string ) const {
	return static_cast < const A * > ( this )->accessComponent < GeneralAlphabet > ( ).get ( ).count ( string );
}

template < >
bool A::Element < A, std::string, SubtreeWildcard >::available ( const std::string & string ) const {
	return static_cast < const A * > ( this )->accessComponent < GeneralAlphabet > ( ).get ( ).count ( string );
}

template < >
void A::Component < A, std::string, GeneralAlphabet >::valid ( const std::string & ) const {
}

template < >
void A::Component < A, std::string, NonlinearAlphabet >::valid ( const std::string & string ) const {
	if ( static_cast < const A * > ( this )->accessElement < SubtreeWildcard > ( ).get ( ) == string )
		throw::exception::CommonException ( "Symbol " + ( std::string ) string + "cannot be set as nonlinear variable since it is already a subtree wildcard" );
}

template < >
void A::Element < A, std::string, SubtreeWildcard >::valid ( const std::string & string ) const {
	if ( static_cast < const A * > ( this )->accessComponent < NonlinearAlphabet > ( ).get ( ).count ( string ) )
		throw::exception::CommonException ( "Symbol " + ( std::string ) string + "cannot be set as subtree wildcard since it is already a nonlinear variable" );
}

}

class B : public std::Components < A, std::string, std::tuple < GeneralAlphabet, NonlinearAlphabet >, std::tuple < > > {
};

namespace std {

template < >
bool B::Component < B, std::string, GeneralAlphabet >::used ( const std::string & ) const {
	return false;
}

template < >
bool B::Component < B, std::string, NonlinearAlphabet >::used ( const std::string & ) const {
	return false;
}

template < >
bool B::Component < B, std::string, GeneralAlphabet >::available ( const std::string & ) const {
	return true;
}

template < >
bool B::Component < B, std::string, NonlinearAlphabet >::available ( const std::string & ) const {
	return true;
}

template < >
void B::Component < B, std::string, GeneralAlphabet >::valid ( const std::string & ) const {
}

template < >
void B::Component < B, std::string, NonlinearAlphabet >::valid ( const std::string & ) const {
}

}

void ComponentsTest::setUp ( ) {
}

void ComponentsTest::tearDown ( ) {
}

void ComponentsTest::testAdd ( ) {
	A tmp ( "2" );

	CPPUNIT_ASSERT_THROW ( tmp.accessComponent < NonlinearAlphabet > ( ).add ( "1" ), exception::CommonException );
	tmp.accessComponent < GeneralAlphabet > ( ).add ( "1" );
	tmp.accessComponent < NonlinearAlphabet > ( ).add ( "1" );
}

void ComponentsTest::testRemove ( ) {
	A tmp ( "2" );

	tmp.accessComponent < GeneralAlphabet > ( ).add ( "1" );
	tmp.accessComponent < NonlinearAlphabet > ( ).add ( "1" );
	CPPUNIT_ASSERT_THROW ( tmp.accessComponent < GeneralAlphabet > ( ).remove ( "1" ), exception::CommonException );
	tmp.accessComponent < NonlinearAlphabet > ( ).remove ( "1" );
	tmp.accessComponent < GeneralAlphabet > ( ).remove ( "1" );
}
