/*
 * BlankSymbol.h
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#ifndef BLANK_SYMBOL_H_
#define BLANK_SYMBOL_H_

#include "SymbolBase.h"
#include "Symbol.h"

namespace alphabet {

/**
 * Represents blank symbol in an alphabet.
 */
class BlankSymbol : public SymbolBase {
public:
	/**
	 * Creates a blank symbol.
	 * @param symbol name of the symbol
	 */
	explicit BlankSymbol ( );

	virtual SymbolBase * clone ( ) const;
	virtual SymbolBase * plunder ( ) &&;

	virtual int compare ( const ObjectBase & other ) const {
		if ( std::type_index ( typeid ( * this ) ) == std::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return std::type_index ( typeid ( * this ) ) - std::type_index ( typeid ( other ) );
	}

	virtual int compare ( const BlankSymbol & other ) const;

	virtual void operator >>( std::ostream & out ) const;

	virtual explicit operator std::string ( ) const;

	static BlankSymbol BLANK;
	static Symbol BLANK_SYMBOL;

	static const std::string & getXmlTagName() {
		static std::string xmlTagName = "BlankSymbol";

		return xmlTagName;
	}

	static BlankSymbol parse ( std::deque < sax::Token >::iterator & input );

	void compose ( std::deque < sax::Token > & out ) const;

	virtual SymbolBase * next ( ) const;
	virtual SymbolBase * inc ( ) &&;
};

} /* namespace alphabet */

#endif /* BLANK_SYMBOL_H_ */
