/*
 * Symbol.h
 *
 *  Created on: Apr 10, 2013
 *      Author: Jan Travnicek
 */

#ifndef SYMBOL_H_
#define SYMBOL_H_

#include <base/WrapperBase.hpp>
#include "SymbolBase.h"

#include <set>
#include <string>

namespace alphabet {

/**
 * Wrapper around automata.
 */
class Symbol : public alib::WrapperBase < SymbolBase > {
	using alib::WrapperBase < SymbolBase >::WrapperBase;

public:
	Symbol next ( ) const;
	void inc ( );

	static const std::string & getXmlTagRefName() {
		static std::string xmlTagName = "SymbolRef";

		return xmlTagName;
	}
};

/**
 * Creates and adds unique state to grammar. If given state name is
 * already used, appends apostrophe or integer suffix
 * @param name name of the state
 * @throws AutomatonException if symbol could not be created
 * @return created symbol
 */
alphabet::Symbol createUniqueSymbol ( alphabet::Symbol base, const std::set < alphabet::Symbol > & terminals, const std::set < alphabet::Symbol > & nonterminals );

alphabet::Symbol symbolFrom ( int number );
alphabet::Symbol symbolFrom ( char character );
alphabet::Symbol symbolFrom ( std::string string );
alphabet::Symbol symbolFrom ( const char * string );

} /* namespace alphabet */

#endif /* SYMBOL_H_ */
