/*
 * UniqueSymbol.cpp
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travicek
 */

#include "UniqueSymbol.h"
#include <sstream>
#include <sax/FromXMLParserHelper.h>
#include "Symbol.h"
#include <object/Object.h>
#include <XmlApi.hpp>

namespace alphabet {

UniqueSymbol::UniqueSymbol(Symbol symbol, primitive::Integer id) : symbol(std::move(symbol)), id(std::move(id)) {

}

SymbolBase* UniqueSymbol::clone() const {
	return new UniqueSymbol(*this);
}

SymbolBase* UniqueSymbol::plunder() && {
	return new UniqueSymbol(std::move(*this));
}

const Symbol& UniqueSymbol::getSymbol() const {
	return symbol;
}

const primitive::Integer& UniqueSymbol::getId() const {
	return id;
}

primitive::Integer& UniqueSymbol::getId() {
	return id;
}

int UniqueSymbol::compare(const UniqueSymbol& other) const {
	int res = symbol.getData().compare(other.symbol.getData());
	if(res == 0) res = id.compare(other.id);
	return res;
}

void UniqueSymbol::operator>>(std::ostream& out) const {
	out << "(UniqueSymbol " << symbol << ", " << id << ")";
}

UniqueSymbol::operator std::string() const {
	std::stringstream ss;
	ss << "{";
	ss << symbol;
	ss << ", ";
	ss << id;
	ss << "}";
	return std::move(ss).str();
}

UniqueSymbol UniqueSymbol::parse(std::deque<sax::Token>::iterator& input) {
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, UniqueSymbol::getXmlTagName());

	Symbol firstSymbol = alib::xmlApi<Symbol>::parse(input);
	primitive::Integer secondSymbol = alib::xmlApi<primitive::Integer>::parse(input);

	UniqueSymbol data(std::move(firstSymbol), std::move(secondSymbol));
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, UniqueSymbol::getXmlTagName());
	return data;
}

void UniqueSymbol::compose(std::deque<sax::Token>& out) const {
	out.emplace_back(UniqueSymbol::getXmlTagName(), sax::Token::TokenType::START_ELEMENT);
	alib::xmlApi<alphabet::Symbol>::compose(out, symbol);
	alib::xmlApi<primitive::Integer>::compose(out, id);
	out.emplace_back(UniqueSymbol::getXmlTagName(), sax::Token::TokenType::END_ELEMENT);
}

SymbolBase* UniqueSymbol::next() const {
	return new UniqueSymbol(symbol, id.next());
}

SymbolBase* UniqueSymbol::inc() && {
	id.inc();
	return NULL;
}

} /* namespace alphabet */

namespace alib {

auto uniqueSymbolParserRegister = xmlApi<alphabet::Symbol>::ParserRegister<alphabet::UniqueSymbol>();
auto uniqueSymbolParserRegister2 = xmlApi<alib::Object>::ParserRegister<alphabet::UniqueSymbol>();

} /* namespace alib */
