/*
 * Deque.cpp
 *
 * Created on: Apr 1, 2013
 * Author: Jan Travnicek
 */

#include "ObjectsDeque.h"

#include "Container.h"
#include <object/Object.h>
#include <sstream>

namespace container {

ObjectsDeque::ObjectsDeque ( std::deque < alib::Object > deque ) : std::deque < alib::Object > ( std::move ( deque ) ) {
}

ObjectsDeque::ObjectsDeque ( ) : std::deque < alib::Object > ( ) {
}

ContainerBase * ObjectsDeque::clone ( ) const {
	return new ObjectsDeque ( * this );
}

ContainerBase * ObjectsDeque::plunder ( ) && {
	return new ObjectsDeque ( std::move ( * this ) );
}

int ObjectsDeque::compare ( const ObjectsDeque & other ) const {
	std::compare < std::deque < alib::Object > > comp;

	return comp ( static_cast < const std::deque < alib::Object > > ( * this ), static_cast < const std::deque < alib::Object > > ( other ) );
}

void ObjectsDeque::operator >>( std::ostream & os ) const {
	os << "(ObjectsDeque " << static_cast < const std::deque < alib::Object > > ( * this ) << ")";
}

ObjectsDeque::operator std::string ( ) const {
	std::stringstream ss;
	ss << * this;
	return std::move ( ss ).str ( );
}

ObjectsDeque ObjectsDeque::parse ( std::deque < sax::Token >::iterator & input ) {
	return ObjectsDeque ( parseRaw < alib::Object > ( input ) );
}

void ObjectsDeque::compose ( std::deque < sax::Token > & out ) const {
	compose ( out, * this );
}

} /* namespace container */

namespace alib {

auto objectsDequeParserRegister = xmlApi < container::Container >::ParserRegister < container::ObjectsDeque > ();
auto objectsDequeParserRegister2 = xmlApi < alib::Object >::ParserRegister < container::ObjectsDeque > ();

} /* namespace alib */
