/*
 * Vector.cpp
 *
 * Created on: Apr 1, 2013
 * Author: Jan Travnicek
 */

#include "ObjectsVector.h"

#include "Container.h"
#include <object/Object.h>
#include <sstream>

namespace container {

ObjectsVector::ObjectsVector ( std::vector < alib::Object > vector ) : std::vector < alib::Object > ( std::move ( vector ) ) {
}

ObjectsVector::ObjectsVector ( ) : std::vector < alib::Object > ( ) {
}

ContainerBase * ObjectsVector::clone ( ) const {
	return new ObjectsVector ( * this );
}

ContainerBase * ObjectsVector::plunder ( ) && {
	return new ObjectsVector ( std::move ( * this ) );
}

int ObjectsVector::compare ( const ObjectsVector & other ) const {
	std::compare < std::vector < alib::Object > > comp;

	return comp ( static_cast < const std::vector < alib::Object > > ( * this ), static_cast < const std::vector < alib::Object > > ( other ) );
}

void ObjectsVector::operator >>( std::ostream & os ) const {
	os << "(ObjectsVector " << static_cast < const std::vector < alib::Object > > ( * this ) << ")";
}

ObjectsVector::operator std::string ( ) const {
	std::stringstream ss;
	ss << * this;
	return std::move ( ss ).str ( );
}

ObjectsVector ObjectsVector::parse ( std::deque < sax::Token >::iterator & input ) {
	return ObjectsVector ( parseRaw < alib::Object > ( input ) );
}

void ObjectsVector::compose ( std::deque < sax::Token > & out ) const {
	compose ( out, * this );
}

} /* namespace container */

namespace alib {

auto objectsVectorParserRegister = xmlApi < container::Container >::ParserRegister < container::ObjectsVector > ();
auto objectsVectorParserRegister2 = xmlApi < alib::Object >::ParserRegister < container::ObjectsVector > ();

} /* namespace alib */
