/*
 * Node.h
 *
 *   Created on: Jun 19, 2014
 *       Author: David Rosca
 *  Modified by: Jan Broz
 */

#ifndef NODE_H_
#define NODE_H_

#include "Macros.h"
#include "../../label/Label.h"

namespace graph {

class Node {
	label::Label name;

	void dump(std::ostream & os) const;

public:
	/// should be used only to create variable, to which will be assigned soon
	Node() : name(label::labelFrom("uninitialized")) {}

	explicit Node(label::Label name);
	explicit Node(int number);
	explicit Node(char character);
	explicit Node(const std::string &name);

	Node * clone() const;
	Node * plunder() &&;

	const label::Label & getName() const;

	bool operator < (const Node & other) const;
	bool operator == (const Node & other) const;
	bool operator != (const Node & other) const;
	int compare(const Node & other) const;

	void operator<<(std::ostream & out) const;
	friend std::ostream & operator<<(std::ostream & out, const Node & node);
	explicit operator std::string() const;
};

} // namespace graph

namespace std {

template<>
struct compare<graph::Node> {
	int operator()(const graph::Node& first, const graph::Node& second) const {
		return first.compare(second);
	}
};

} /* namespace std */

GRAPH_DEFINE_STD_HASH(Node)

#endif // NODE_H_
