/*
 * ObjectLabel.cpp
 *
 *  Created on: Mar 26, 2013
 *      Author:Jan Travnicek 
 */

#include "ObjectLabel.h"
#include <sax/FromXMLParserHelper.h>
#include "Label.h"
#include <object/Object.h>
#include <XmlApi.hpp>
#include "UniqueLabel.h"

namespace label {

ObjectLabel::ObjectLabel(alib::Object label) : label(std::move(label)) {

}

LabelBase* ObjectLabel::clone() const {
	return new ObjectLabel(*this);
}

LabelBase* ObjectLabel::plunder() && {
	return new ObjectLabel(std::move(*this));
}

const alib::Object& ObjectLabel::getData() const {
	return label;
}

int ObjectLabel::compare(const ObjectLabel& other) const {
	return label.getData().compare(other.label.getData());
}

void ObjectLabel::operator>>(std::ostream& out) const {
	out << "(ObjectLabel " << label << ")";
}

ObjectLabel::operator std::string() const {
	return (std::string) label;
}

ObjectLabel ObjectLabel::parse(std::deque<sax::Token>::iterator& input) {
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, ObjectLabel::getXmlTagName());
	ObjectLabel data(alib::xmlApi<alib::Object>::parse(input));
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, ObjectLabel::getXmlTagName());
	return data;
}

void ObjectLabel::compose(std::deque<sax::Token>& out) const {
	out.emplace_back(ObjectLabel::getXmlTagName(), sax::Token::TokenType::START_ELEMENT);
	alib::xmlApi<alib::Object>::compose(out, label);
	out.emplace_back(ObjectLabel::getXmlTagName(), sax::Token::TokenType::END_ELEMENT);
}

LabelBase* ObjectLabel::next() const {
	return new UniqueLabel(Label(*this), primitive::Integer(0));
}

LabelBase* ObjectLabel::inc() && {
	return new UniqueLabel(Label(std::move(*this)), primitive::Integer(0));
}

} /* namespace label */

namespace alib {

auto objectLabelParserRegister = xmlApi<label::Label>::ParserRegister<label::ObjectLabel>();
auto objectLabelParserRegister2 = xmlApi<alib::Object>::ParserRegister<label::ObjectLabel>();

} /* namespace alib */
