/*
 * FormalRegExpIteration.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Martin Zak
 */

#include "FormalRegExpIteration.h"
#include <exception/CommonException.h>

#include "../unbounded/UnboundedRegExpIteration.h"
#include <sstream>

namespace regexp {

FormalRegExpIteration::FormalRegExpIteration ( FormalRegExpElement && element ) : element ( nullptr ) {
	setElement ( std::move ( element ) );
}

FormalRegExpIteration::FormalRegExpIteration ( const FormalRegExpElement & element ) : FormalRegExpIteration ( std::move_copy ( element ) ) {
}

FormalRegExpIteration::FormalRegExpIteration ( const FormalRegExpIteration & other ) : element ( other.element ) {
}

FormalRegExpIteration::FormalRegExpIteration ( FormalRegExpIteration && other ) noexcept : element ( std::move ( other.element ) ) {
	element->attachRegExp ( NULL );
}

FormalRegExpIteration & FormalRegExpIteration::operator =( const FormalRegExpIteration & other ) {
	if ( this == & other )
		return * this;

	* this = FormalRegExpIteration ( other );

	return * this;
}

FormalRegExpIteration & FormalRegExpIteration::operator =( FormalRegExpIteration && other ) noexcept {
	std::swap ( this->element, other.element );
	std::swap ( this->parentRegExp, other.parentRegExp );

	this->attachRegExp ( other.parentRegExp );

	return * this;
}

regexp::FormalRegExpIteration::~FormalRegExpIteration ( ) noexcept {
}

const FormalRegExpElement & FormalRegExpIteration::getElement ( ) const {
	return * element;
}

FormalRegExpElement & FormalRegExpIteration::getElement ( ) {
	return * element;
}

void FormalRegExpIteration::setElement ( FormalRegExpElement && elementParam ) {
	element = std::smart_ptr < FormalRegExpElement > ( std::move ( elementParam ).plunder ( ) );

	if ( this->parentRegExp && !element->attachRegExp ( this->parentRegExp ) )
		throw exception::CommonException ( "Input symbols not in the alphabet." );
}

void FormalRegExpIteration::setElement ( const FormalRegExpElement & elementParam ) {
	setElement ( std::move_copy ( elementParam ) );
}

FormalRegExpElement * FormalRegExpIteration::clone ( ) const {
	return new FormalRegExpIteration ( * this );
}

FormalRegExpElement * FormalRegExpIteration::plunder ( ) && {
	return new FormalRegExpIteration ( std::move ( * this ) );
}

std::smart_ptr < UnboundedRegExpElement > FormalRegExpIteration::asUnbounded ( ) const {
	return std::smart_ptr < UnboundedRegExpElement > ( new UnboundedRegExpIteration ( * element->asUnbounded ( ) ) );
}

int FormalRegExpIteration::compare ( const FormalRegExpIteration & other ) const {
	return element->compare ( * other.element );
}

void FormalRegExpIteration::operator >>( std::ostream & out ) const {
	out << "(RegExpFormalRegExpIteration " << * element << ")";
}

bool FormalRegExpIteration::testSymbol ( const alphabet::Symbol & symbol ) const {
	return element->testSymbol ( symbol );
}

bool FormalRegExpIteration::attachRegExp ( const FormalRegExp * regexp ) {
	if ( this->parentRegExp == regexp ) return true;

	this->parentRegExp = regexp;
	return this->element->attachRegExp ( regexp );
}

void FormalRegExpIteration::computeMinimalAlphabet ( std::set < alphabet::Symbol > & alphabet ) const {
	element->computeMinimalAlphabet ( alphabet );
}

FormalRegExpIteration::operator std::string ( ) const {
	std::stringstream ss;
	ss << * this;
	return ss.str ( );
}

} /* namespace regexp */
