/*
 * FormalRegExpSymbol.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Martin Zak
 */

#include "FormalRegExpSymbol.h"
#include "../unbounded/UnboundedRegExpSymbol.h"
#include <sstream>

namespace regexp {

FormalRegExpSymbol::FormalRegExpSymbol ( int number ) : symbol ( alphabet::symbolFrom ( number ) ) {
}

FormalRegExpSymbol::FormalRegExpSymbol ( char character ) : symbol ( alphabet::symbolFrom ( character ) ) {
}

FormalRegExpSymbol::FormalRegExpSymbol ( const std::string & label ) : symbol ( alphabet::symbolFrom ( label ) ) {
}

FormalRegExpSymbol::FormalRegExpSymbol ( alphabet::Symbol symbol ) : symbol ( std::move ( symbol ) ) {
}

FormalRegExpSymbol::FormalRegExpSymbol ( const FormalRegExpSymbol & other ) : symbol ( other.symbol ) {
}

FormalRegExpSymbol::FormalRegExpSymbol ( FormalRegExpSymbol && other ) noexcept : symbol ( std::move ( other.symbol ) ) {
}

FormalRegExpSymbol & FormalRegExpSymbol::operator =( const FormalRegExpSymbol & other ) {
	if ( this == & other )
		return * this;

	* this = FormalRegExpSymbol ( other );

	return * this;
}

FormalRegExpSymbol & FormalRegExpSymbol::operator =( FormalRegExpSymbol && other ) noexcept {
	std::swap ( this->symbol, other.symbol );
	std::swap ( this->parentRegExp, other.parentRegExp );

	this->attachRegExp ( other.parentRegExp );

	return * this;
}

FormalRegExpElement * FormalRegExpSymbol::clone ( ) const {
	return new FormalRegExpSymbol ( * this );
}

FormalRegExpElement * FormalRegExpSymbol::plunder ( ) && {
	return new FormalRegExpSymbol ( std::move ( * this ) );
}

std::smart_ptr < UnboundedRegExpElement > FormalRegExpSymbol::asUnbounded ( ) const {
	return std::smart_ptr < UnboundedRegExpElement > ( new UnboundedRegExpSymbol ( this->symbol ) );
}

int FormalRegExpSymbol::compare ( const FormalRegExpSymbol & other ) const {
	return symbol.getData ( ).compare ( other.symbol.getData ( ) );
}

void FormalRegExpSymbol::operator >>( std::ostream & out ) const {
	out << "(FormalRegExpSymbol " << symbol << ")";
}

bool FormalRegExpSymbol::testSymbol ( const alphabet::Symbol & symbol ) const {
	return symbol == this->symbol;
}

bool FormalRegExpSymbol::attachRegExp ( const FormalRegExp * regexp ) {
	if ( this->parentRegExp == regexp ) return true;

	this->parentRegExp = regexp;

	if ( regexp == NULL ) return true;

	return this->parentRegExp->getAlphabet ( ).find ( this->symbol ) != this->parentRegExp->getAlphabet ( ).end ( );
}

void FormalRegExpSymbol::computeMinimalAlphabet ( std::set < alphabet::Symbol > & alphabet ) const {
	alphabet.insert ( this->symbol );
}

const alphabet::Symbol & FormalRegExpSymbol::getSymbol ( ) const {
	return this->symbol;
}

FormalRegExpSymbol::operator std::string ( ) const {
	std::stringstream ss;
	ss << * this;
	return ss.str ( );
}

} /* namespace regexp */
