/*
 * UnboundedRegExpConcatenation.h
 *
 *  Created on: Nov 27, 2013
 *      Author: Martin Zak
 */

#ifndef UNBOUNDED_REG_EXP_CONCATENATION_H_
#define UNBOUNDED_REG_EXP_CONCATENATION_H_

#include <vector>
#include "UnboundedRegExpElement.h"

namespace regexp {

namespace simplify {

class RegExpOptimize;

} /* namespace simplify */

/**
 * Represents concatenation operator in the regular expression. Contains list of UnboundedRegExpElement
 * as operands of the operator.
 */
class UnboundedRegExpConcatenation : public UnboundedRegExpElement {
protected:
	std::vector < std::smart_ptr < UnboundedRegExpElement > > elements;

	/**
	 * @copydoc UnboundedRegExpElement::attachRegExp()
	 */
	virtual bool attachRegExp ( const UnboundedRegExp * regexp );

public:
	void Accept ( void * userData, const UnboundedRegExpElementVisitor & visitor ) const {
		visitor.Visit ( userData, * this );
	}

	explicit UnboundedRegExpConcatenation ( );

	UnboundedRegExpConcatenation ( const UnboundedRegExpConcatenation & other );
	UnboundedRegExpConcatenation ( UnboundedRegExpConcatenation && other ) noexcept;
	UnboundedRegExpConcatenation & operator =( const UnboundedRegExpConcatenation & other );
	UnboundedRegExpConcatenation & operator =( UnboundedRegExpConcatenation && other ) noexcept;
	virtual ~UnboundedRegExpConcatenation ( ) noexcept;

	/**
	 * @copydoc UnboundedRegExpElement::clone() const
	 */
	virtual UnboundedRegExpElement * clone ( ) const;

	/**
	 * @copydoc UnboundedRegExpElement::plunder() const
	 */
	virtual UnboundedRegExpElement * plunder ( ) &&;

	/**
	 * @copydoc UnboundedRegExpElement::cloneAsFormal() const
	 */
	virtual std::smart_ptr < FormalRegExpElement > asFormal ( ) const;

	/**
	 * @copydoc UnboundedRegExpElement::testSymbol() const
	 */
	virtual bool testSymbol ( const alphabet::Symbol & symbol ) const;

	/**
	 * @copydoc UnboundedRegExpElement::computeMinimalAlphabet()
	 */
	virtual void computeMinimalAlphabet ( std::set < alphabet::Symbol > & alphabet ) const;

	/**
	 * @return elements
	 */
	const std::vector < std::smart_ptr < const UnboundedRegExpElement > > & getElements ( ) const;

	/**
	 * @return elements
	 */
	const std::vector < std::smart_ptr < UnboundedRegExpElement > > & getElements ( );

	/**
	 * @param element to append
	 */
	void appendElement ( UnboundedRegExpElement && element );
	void appendElement ( const UnboundedRegExpElement & element );

	virtual int compare ( const UnboundedRegExpElement & other ) const {
		if ( std::type_index ( typeid ( * this ) ) == std::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return std::type_index ( typeid ( * this ) ) - std::type_index ( typeid ( other ) );
	}

	virtual int compare ( const UnboundedRegExpConcatenation & ) const;

	/**
	 * @copydoc UnboundedRegExpElement::operator>>() const
	 */
	virtual void operator >>( std::ostream & out ) const;

	friend class simplify::RegExpOptimize;

	virtual explicit operator std::string ( ) const;
};

} /* namespace regexp */

#endif /* UNBOUNDED_REG_EXP_CONCATENATION_H_ */
