/*
 * UnboundedRegExpIteration.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Martin Zak
 */

#include "UnboundedRegExpIteration.h"
#include <exception/CommonException.h>

#include "../formal/FormalRegExpIteration.h"
#include <sstream>

namespace regexp {

UnboundedRegExpIteration::UnboundedRegExpIteration ( UnboundedRegExpElement && element ) : element ( nullptr ) {
	setElement ( std::move ( element ) );
}

UnboundedRegExpIteration::UnboundedRegExpIteration ( const UnboundedRegExpElement & element ) : UnboundedRegExpIteration ( std::move_copy ( element ) ) {
}

UnboundedRegExpIteration::UnboundedRegExpIteration ( const UnboundedRegExpIteration & other ) : element ( other.element ) {
}

UnboundedRegExpIteration::UnboundedRegExpIteration ( UnboundedRegExpIteration && other ) noexcept : element ( std::move ( other.element ) ) {
	element->attachRegExp ( NULL );
}

UnboundedRegExpIteration & UnboundedRegExpIteration::operator =( const UnboundedRegExpIteration & other ) {
	if ( this == & other )
		return * this;

	* this = UnboundedRegExpIteration ( other );

	return * this;
}

UnboundedRegExpIteration & UnboundedRegExpIteration::operator =( UnboundedRegExpIteration && other ) noexcept {
	std::swap ( this->element, other.element );
	std::swap ( this->parentRegExp, other.parentRegExp );

	this->attachRegExp ( other.parentRegExp );

	return * this;
}

UnboundedRegExpIteration::~UnboundedRegExpIteration ( ) noexcept {
}

const UnboundedRegExpElement & UnboundedRegExpIteration::getElement ( ) const {
	return * element;
}

UnboundedRegExpElement & UnboundedRegExpIteration::getElement ( ) {
	return * element;
}

void UnboundedRegExpIteration::setElement ( UnboundedRegExpElement && elementParam ) {
	element = std::smart_ptr < UnboundedRegExpElement > ( std::move ( elementParam ).plunder ( ) );

	if ( this->parentRegExp && !element->attachRegExp ( this->parentRegExp ) )
		throw exception::CommonException ( "Input symbols not in the alphabet." );
}

void UnboundedRegExpIteration::setElement ( const UnboundedRegExpElement & elementParam ) {
	setElement ( std::move_copy ( elementParam ) );
}

UnboundedRegExpElement * UnboundedRegExpIteration::clone ( ) const {
	return new UnboundedRegExpIteration ( * this );
}

UnboundedRegExpElement * UnboundedRegExpIteration::plunder ( ) && {
	return new UnboundedRegExpIteration ( std::move ( * this ) );
}

std::smart_ptr < FormalRegExpElement > UnboundedRegExpIteration::asFormal ( ) const {
	return std::smart_ptr < FormalRegExpElement > ( new FormalRegExpIteration ( std::move ( * element->asFormal ( ) ) ) );
}

int UnboundedRegExpIteration::compare ( const UnboundedRegExpIteration & other ) const {
	return element->compare ( * other.element );
}

void UnboundedRegExpIteration::operator >>( std::ostream & out ) const {
	out << "(RegExpUnboundedRegExpIteration " << * element << ")";
}

bool UnboundedRegExpIteration::testSymbol ( const alphabet::Symbol & symbol ) const {
	return element->testSymbol ( symbol );
}

bool UnboundedRegExpIteration::attachRegExp ( const UnboundedRegExp * regexp ) {
	if ( this->parentRegExp == regexp ) return true;

	this->parentRegExp = regexp;
	return this->element->attachRegExp ( regexp );
}

void UnboundedRegExpIteration::computeMinimalAlphabet ( std::set < alphabet::Symbol > & alphabet ) const {
	element->computeMinimalAlphabet ( alphabet );
}

UnboundedRegExpIteration::operator std::string ( ) const {
	std::stringstream ss;
	ss << * this;
	return ss.str ( );
}

} /* namespace regexp */
