/*
 * LinearString.h
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#ifndef LINEAR_STRING_H_
#define LINEAR_STRING_H_

#include "StringBase.h"
#include <set>
#include <vector>

#include "../alphabet/Symbol.h"
#include <core/components.hpp>

namespace tree {

class PrefixRankedTree;
class PrefixRankedPattern;
class PrefixRankedBarTree;
class PrefixRankedBarPattern;
class PrefixBarTree;

}

namespace string {

class Epsilon;
class GeneralAlphabet;

/**
 * Represents regular expression parsed from the XML. Regular expression is stored
 * as a tree of LinearStringElement.
 */
class LinearString : public StringBase, public std::Components < LinearString, alphabet::Symbol, std::tuple < GeneralAlphabet >, std::tuple < > > {
	std::vector < alphabet::Symbol > m_Data;

public:
	explicit LinearString ( );
	explicit LinearString ( const tree::PrefixRankedTree & tree );
	explicit LinearString ( const tree::PrefixRankedPattern & tree );
	explicit LinearString ( const tree::PrefixRankedBarTree & tree );
	explicit LinearString ( const tree::PrefixRankedBarPattern & tree );
	explicit LinearString ( const tree::PrefixBarTree & tree );
	explicit LinearString ( std::set < alphabet::Symbol > alphabet, std::vector < alphabet::Symbol > data );
	explicit LinearString ( std::vector < alphabet::Symbol > data );
	explicit LinearString ( const std::string & string );
	explicit LinearString ( const char * string );
	explicit LinearString ( const Epsilon & epsilon );

	virtual StringBase * clone ( ) const;
	virtual StringBase * plunder ( ) &&;

	virtual const std::set < alphabet::Symbol > & getAlphabet ( ) const {
		return accessComponent < GeneralAlphabet > ( ).get ( );
	}

	void extendAlphabet ( const std::set < alphabet::Symbol > & symbols ) {
		accessComponent < GeneralAlphabet > ( ).add( symbols );
	}

	/**
	 * @param element to append
	 */
	void appendSymbol ( alphabet::Symbol symbol );

	/**
	 * @return List of symbols forming string (const version).
	 */
	const std::vector < alphabet::Symbol > & getContent ( ) const;

	void setContent ( std::vector < alphabet::Symbol > data );

	/**
	 * @return true if string is an empty word (vector length is 0)
	 */
	bool isEmpty ( ) const;

	virtual int compare ( const ObjectBase & other ) const {
		if ( std::type_index ( typeid ( * this ) ) == std::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return std::type_index ( typeid ( * this ) ) - std::type_index ( typeid ( other ) );
	}

	virtual int compare ( const LinearString & other ) const;

	virtual void operator >>( std::ostream & out ) const;

	virtual explicit operator std::string ( ) const;

	static const std::string & getXmlTagName() {
		static std::string xmlTagName = "LinearString";

		return xmlTagName;
	}

	static LinearString parse ( std::deque < sax::Token >::iterator & input );

	void compose ( std::deque < sax::Token > & out ) const;
};

} /* namespace string */

#endif /* LINEAR_STRING_H_ */
