/*
 * UselessStatesRemover.cpp
 *
 *  Created on: 23. 3. 2014
 *	  Author: Tomas Pecka
 */

#include "UselessStatesRemover.h"

#include <automaton/FSM/ExtendedNFA.h>
#include <automaton/FSM/CompactNFA.h>
#include <automaton/FSM/EpsilonNFA.h>
#include <automaton/FSM/MultiInitialStateNFA.h>
#include <automaton/FSM/NFA.h>
#include <automaton/FSM/DFA.h>

#include "../../properties/efficient/UsefullStates.h"

#include "automaton/common/State.h"
#include "automaton/Automaton.h"

namespace automaton {

namespace simplify {

namespace efficient {

template<class T>
T UselessStatesRemover::remove( const T & fsm ) {
	// 1.
	std::set<automaton::State> Qu = automaton::properties::efficient::UsefullStates::usefullStates( fsm );

	// 2.
	T M(fsm.getInitialState());

	for( const auto & a : fsm.getInputAlphabet( ) )
		M.addInputSymbol( a );

	if(Qu.size() == 0) {
		return M;
	}

	for( const auto & q : Qu )
		M.addState( q );

	for( const auto & t : fsm.getTransitions( ) )
		for( const auto & to : t.second )
			if( /* this part is not needed Qu.count(t.first.first) ) && */ Qu.count(to) )
				M.addTransition( t.first.first, t.first.second, to );

	for( const auto & q : fsm.getFinalStates( ) )
		M.addFinalState( q );

	return M;
}

auto UselessStatesRemoverEpsilonNFA = UselessStatesRemover::RegistratorWrapper<automaton::EpsilonNFA, automaton::EpsilonNFA>(UselessStatesRemover::remove);
auto UselessStatesRemoverNFA = UselessStatesRemover::RegistratorWrapper<automaton::NFA, automaton::NFA>(UselessStatesRemover::remove);
auto UselessStatesRemoverCompactNFA = UselessStatesRemover::RegistratorWrapper<automaton::CompactNFA, automaton::CompactNFA>(UselessStatesRemover::remove);
auto UselessStatesRemoverExtendedNFA = UselessStatesRemover::RegistratorWrapper<automaton::ExtendedNFA, automaton::ExtendedNFA>(UselessStatesRemover::remove);

template<>
automaton::DFA UselessStatesRemover::remove( const automaton::DFA & fsm ) {
	// 1.
	std::set<automaton::State> Qu = automaton::properties::efficient::UsefullStates::usefullStates( fsm );

	// 2.
	automaton::DFA M ( fsm.getInitialState () );

	for( const auto & a : fsm.getInputAlphabet( ) )
		M.addInputSymbol( a );

	if(Qu.size() == 0) {
		return M;
	}

	for( const auto & q : Qu )
		M.addState( q );

	for( const auto & t : fsm.getTransitions( ) )
		if( /* this part is not needed Qu.count( t.first.first ) && */ Qu.count( t.second ) )
			M.addTransition( t.first.first, t.first.second, t.second );

	for( const auto & q : fsm.getFinalStates( ) )
		M.addFinalState( q );

	return M;
}

auto UselessStatesRemoverDFA = UselessStatesRemover::RegistratorWrapper<automaton::DFA, automaton::DFA>(UselessStatesRemover::remove);

template<>
automaton::MultiInitialStateNFA UselessStatesRemover::remove( const automaton::MultiInitialStateNFA & fsm ) {
	// 1.
	std::set<automaton::State> Qu = automaton::properties::efficient::UsefullStates::usefullStates( fsm );

	// 2.
	automaton::MultiInitialStateNFA M;

	for( const auto & a : fsm.getInputAlphabet( ) )
		M.addInputSymbol( a );

	if(Qu.size() == 0) {
		return M;
	}

	for( const auto & q : Qu )
		M.addState( q );

	for(const auto& init : fsm.getInitialStates()) {
		if( Qu.count(init) )
			M.addInitialState( init );
	}

	for( const auto & t : fsm.getTransitions( ) )
		for( const auto & to : t.second )
			if( Qu.count(to) )
				M.addTransition( t.first.first, t.first.second, to );

	for( const auto & q : fsm.getFinalStates( ) )
		M.addFinalState( q );

	return M;
}

auto UselessStatesRemoverMultiInitialStateNFA = UselessStatesRemover::RegistratorWrapper<automaton::MultiInitialStateNFA, automaton::MultiInitialStateNFA>(UselessStatesRemover::remove);

automaton::Automaton UselessStatesRemover::remove(const automaton::Automaton& automaton) {
	return dispatch(automaton.getData());
}

} /* namespace efficient */

} /* namespace simplify */

} /* namespace automaton */
