#ifndef GRAMMAR_TO_STRING_COMPOSER_H_
#define GRAMMAR_TO_STRING_COMPOSER_H_

#include <ostream>
#include <core/multipleDispatch.hpp>
#include "grammar/Grammar.h"
#include "grammar/GrammarFeatures.h"

namespace grammar {

class GrammarToStringComposer: public std::SingleDispatchFirstStaticParam<GrammarToStringComposer, void, std::ostream&, GrammarBase> {
public:
	template<class T>
	static void composeCFLikeGrammar(std::ostream& output, const T& grammar);
	template<class T>
	static void composeCSLikeGrammar(std::ostream& output, const T& grammar);
	template<class T>
	static void composePreservingCSLikeGrammar(std::ostream& output, const T& grammar);

	/**
	 * Prints XML representation of Automaton to the output stream.
	 * @param automaton automaton to print
	 * @return list of xml tokens representing the automaton
	 */
	static void compose(std::ostream& output, const Grammar& grammar);

	static void compose(std::ostream& output, const LeftLG& grammar);
	static void compose(std::ostream& output, const LeftRG& grammar);
	static void compose(std::ostream& output, const RightLG& grammar);
	static void compose(std::ostream& output, const RightRG& grammar);
	static void compose(std::ostream& output, const LG& grammar);
	static void compose(std::ostream& output, const CFG& grammar);
	static void compose(std::ostream& output, const EpsilonFreeCFG& grammar);
	static void compose(std::ostream& output, const CNF& grammar);
	static void compose(std::ostream& output, const GNF& grammar);
	static void compose(std::ostream& output, const CSG& grammar);
	static void compose(std::ostream& output, const NonContractingGrammar& grammar);
	static void compose(std::ostream& output, const ContextPreservingUnrestrictedGrammar& grammar);
	static void compose(std::ostream& output, const UnrestrictedGrammar& grammar);
};

}

#endif /* GRAMMAR_TO_STRING_COMPOSER_H_ */
