/*
 * RegExpToStringComposer.h
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#ifndef REX_EXP_TO_STRING_COMPOSER_H_
#define REX_EXP_TO_STRING_COMPOSER_H_

#include <string>
#include <core/multipleDispatch.hpp>
#include "regexp/RegExp.h"
#include "regexp/unbounded/UnboundedRegExpElement.h"
#include "regexp/formal/FormalRegExpElement.h"

namespace regexp {

class RegExpToStringComposer : public std::SingleDispatchFirstStaticParam<RegExpToStringComposer, void, std::ostream&, RegExpBase>, UnboundedRegExpElementVisitor, FormalRegExpElementVisitor {
private:
	void Visit(void*, const UnboundedRegExpAlternation& alternation) const;
	void Visit(void*, const UnboundedRegExpConcatenation& concatenation) const;
	void Visit(void*, const UnboundedRegExpIteration& iteration) const;
	void Visit(void*, const UnboundedRegExpSymbol& symbol) const;
	void Visit(void*, const UnboundedRegExpEpsilon& epsilon) const;
	void Visit(void*, const UnboundedRegExpEmpty& empty) const;

	void Visit(void*, const FormalRegExpAlternation& alternation) const;
	void Visit(void*, const FormalRegExpConcatenation& concatenation) const;
	void Visit(void*, const FormalRegExpIteration& iteration) const;
	void Visit(void*, const FormalRegExpSymbol& symbol) const;
	void Visit(void*, const FormalRegExpEpsilon& epsilon) const;
	void Visit(void*, const FormalRegExpEmpty& empty) const;

	enum class Priority {
		ALTERNATION,
		CONCATENATION,
		FACTOR
	};

public:
	/**
	 * Composes string representation of RegExp.
	 * @param regexp RegExp to print
	 * @returns string representation of regexp
	 */
	static void compose(std::ostream& out, const RegExp& regexp);

	static void compose(std::ostream& out, const UnboundedRegExp& regexp);
	static void compose(std::ostream& out, const FormalRegExp& regexp);
};

} /* namespace regexp */

#endif /* REX_EXP_TO_STRING_COMPOSER_H_ */
