/*
 * astringology.cpp
 *
 *  Created on: 26. 3. 2014
 *	  Author: Jan Travnicek
 */

#include <tclap/CmdLine.h>
#include <global/GlobalData.h>
#include <measure>
#include <vector>
#include <sax/FromXMLParserHelper.h>

#include <factory/XmlDataFactory.hpp>
#include <exception/CommonException.h>
#include <string/String.h>
#include <automaton/Automaton.h>
#include <container/Container.h>
#include <container/ObjectsSet.h>
#include <container/ObjectsVector.h>

#include <stringology/exact/ExactFactorMatch.h>
#include <stringology/exact/BoyerMooreHorspool.h>
#include <stringology/exact/ReversedBoyerMooreHorspool.h>
#include <stringology/exact/DeadZoneUsingBadCharacterShift.h>
#include <stringology/exact/BackwardNondeterministicDAWGMatching.hpp>
#include <stringology/exact/BackwardOracleMatching.h>
#include <stringology/exact/BackwardDAWGMatching.h>
#include <stringology/exact/ExactMatchingAutomaton.h>
#include <stringology/exact/ExactFactorAutomaton.h>
#include <stringology/exact/ExactSubsequenceAutomaton.h>
#include <stringology/exact/ExactNondeterministicSubsequenceAutomaton.h>
#include <stringology/exact/ExactMultiNondeterministicSubsequenceAutomaton.h>
#include <stringology/exact/FactorOracleAutomaton.h>
#include <stringology/exact/SuffixAutomaton.h>
#include <stringology/exact/BorderArray.h>
#include <stringology/indexing/SuffixTrie.h>

int main ( int argc, char * argv[] ) {
	try {
		common::GlobalData::argc = argc;
		common::GlobalData::argv = argv;

		TCLAP::CmdLine cmd ( "Stringology algorithm access binary", ' ', "0.01" );

		std::vector < std::string > allowed;
		allowed.push_back ( "exactMatchingAutomaton" );
		allowed.push_back ( "exactFactorAutomaton" );
		allowed.push_back ( "exactSubsequenceAutomaton" );
		allowed.push_back ( "exactNondeterministicSubsequenceAutomaton" );
		allowed.push_back ( "exactMultiNondeterministicSubsequenceAutomaton" );
		allowed.push_back ( "factorOracleAutomaton" );
		allowed.push_back ( "suffixAutomaton" );
		allowed.push_back ( "exactFactorMatch" );
		allowed.push_back ( "boyerMooreHorspool" );
		allowed.push_back ( "reversedBoyerMooreHorspool" );
		allowed.push_back ( "deadZoneUsingBadCharacterShift" );
		allowed.push_back ( "backwardNondeterministicDAWGMatching" );
		allowed.push_back ( "backwardOracleMatching" );
		allowed.push_back ( "backwardDAWGMatching" );
		allowed.push_back ( "borderArray" );
		allowed.push_back ( "suffixTrie" );
		TCLAP::ValuesConstraint < std::string > allowedVals ( allowed );

		TCLAP::ValueArg < std::string > algorithm ( "a", "algorithm", "Execute algorithm", false, "exactFactorMatch", & allowedVals );
		cmd.add ( algorithm );

		TCLAP::MultiArg < std::string > subjectInput ( "s", "subject", "Subject string from file", false, "file" );
		cmd.add ( subjectInput );

		TCLAP::MultiArg < std::string > patternInput ( "p", "pattern", "Pattern string from file", false, "file" );
		cmd.add ( patternInput );

		TCLAP::SwitchArg measure ( "m", "measure", "Measure times", false );
		cmd.add ( measure );

		TCLAP::SwitchArg verbose ( "v", "verbose", "Be verbose", false );
		cmd.add ( verbose );

		cmd.parse ( argc, argv );

		if(verbose.isSet())
			common::GlobalData::verbose = true;
		if(measure.isSet())
			common::GlobalData::measure = true;

		measurements::start ( "Overal", measurements::Type::OVERALL );
		measurements::start ( "Input read", measurements::Type::AUXILIARY );

		if ( algorithm.getValue ( ) == "exactFactorMatch" ) {
			string::String subject = alib::XmlDataFactory::fromTokens < string::String > ( std::move ( sax::FromXMLParserHelper::parseInput(true, subjectInput).front ( ) ) );
			string::String pattern = alib::XmlDataFactory::fromTokens < string::String > ( std::move ( sax::FromXMLParserHelper::parseInput(true, patternInput).front ( ) ) );

			measurements::end ( );
			measurements::start ( "Algorithm", measurements::Type::MAIN );

			std::set < unsigned > res = stringology::exact::ExactFactorMatch::match ( subject, pattern );

			measurements::end ( );
			measurements::start ( "Output write", measurements::Type::AUXILIARY );

			alib::XmlDataFactory::toStdout ( res );
		} else if ( algorithm.getValue ( ) == "boyerMooreHorspool" ) {
			string::String subject = alib::XmlDataFactory::fromTokens < string::String > ( std::move ( sax::FromXMLParserHelper::parseInput(true, subjectInput).front ( ) ) );
			string::String pattern = alib::XmlDataFactory::fromTokens < string::String > ( std::move ( sax::FromXMLParserHelper::parseInput(true, patternInput).front ( ) ) );

			measurements::end ( );
			measurements::start ( "Algorithm", measurements::Type::MAIN );

			std::set < unsigned > res = stringology::exact::BoyerMooreHorspool::match ( subject, pattern );

			measurements::end ( );
			measurements::start ( "Output write", measurements::Type::AUXILIARY );

			alib::XmlDataFactory::toStdout ( res );
		} else if ( algorithm.getValue ( ) == "reversedBoyerMooreHorspool" ) {
			string::String subject = alib::XmlDataFactory::fromTokens < string::String > ( std::move ( sax::FromXMLParserHelper::parseInput(true, subjectInput).front ( ) ) );
			string::String pattern = alib::XmlDataFactory::fromTokens < string::String > ( std::move ( sax::FromXMLParserHelper::parseInput(true, patternInput).front ( ) ) );

			measurements::end ( );
			measurements::start ( "Algorithm", measurements::Type::MAIN );

			std::set < unsigned > res = stringology::exact::ReversedBoyerMooreHorspool::match ( subject, pattern );

			measurements::end ( );
			measurements::start ( "Output write", measurements::Type::AUXILIARY );

			alib::XmlDataFactory::toStdout ( res );
		} else if ( algorithm.getValue ( ) == "deadZoneUsingBadCharacterShift" ) {
			string::String subject = alib::XmlDataFactory::fromTokens < string::String > ( std::move ( sax::FromXMLParserHelper::parseInput(true, subjectInput).front ( ) ) );
			string::String pattern = alib::XmlDataFactory::fromTokens < string::String > ( std::move ( sax::FromXMLParserHelper::parseInput(true, patternInput).front ( ) ) );

			measurements::end ( );
			measurements::start ( "Algorithm", measurements::Type::MAIN );

			std::set < unsigned > res = stringology::exact::DeadZoneUsingBadCharacterShift::match ( subject, pattern );

			measurements::end ( );
			measurements::start ( "Output write", measurements::Type::AUXILIARY );

			alib::XmlDataFactory::toStdout ( res );
		} else if ( algorithm.getValue ( ) == "backwardNondeterministicDAWGMatching" ) {
			string::String subject = alib::XmlDataFactory::fromTokens < string::String > ( std::move ( sax::FromXMLParserHelper::parseInput(true, subjectInput).front ( ) ) );
			string::String pattern = alib::XmlDataFactory::fromTokens < string::String > ( std::move ( sax::FromXMLParserHelper::parseInput(true, patternInput).front ( ) ) );

			measurements::end ( );
			measurements::start ( "Algorithm", measurements::Type::MAIN );

			std::set < unsigned > res = stringology::exact::BackwardNondeterministicDAWGMatching::match ( subject, pattern );

			measurements::end ( );
			measurements::start ( "Output write", measurements::Type::AUXILIARY );

			alib::XmlDataFactory::toStdout ( res );
		} else if ( algorithm.getValue ( ) == "backwardOracleMatching" ) {
			string::String subject = alib::XmlDataFactory::fromTokens < string::String > ( std::move ( sax::FromXMLParserHelper::parseInput(true, subjectInput).front ( ) ) );
			string::String pattern = alib::XmlDataFactory::fromTokens < string::String > ( std::move ( sax::FromXMLParserHelper::parseInput(true, patternInput).front ( ) ) );

			measurements::end ( );
			measurements::start ( "Algorithm", measurements::Type::MAIN );

			std::set < unsigned > res = stringology::exact::BackwardOracleMatching::match ( subject, pattern );

			measurements::end ( );
			measurements::start ( "Output write", measurements::Type::AUXILIARY );

			alib::XmlDataFactory::toStdout ( res );
		} else if ( algorithm.getValue ( ) == "backwardDAWGMatching" ) {
			string::String subject = alib::XmlDataFactory::fromTokens < string::String > ( std::move ( sax::FromXMLParserHelper::parseInput(true, subjectInput).front ( ) ) );
			string::String pattern = alib::XmlDataFactory::fromTokens < string::String > ( std::move ( sax::FromXMLParserHelper::parseInput(true, patternInput).front ( ) ) );

			measurements::end ( );
			measurements::start ( "Algorithm", measurements::Type::MAIN );

			std::set < unsigned > res = stringology::exact::BackwardDAWGMatching::match ( subject, pattern );

			measurements::end ( );
			measurements::start ( "Output write", measurements::Type::AUXILIARY );

			alib::XmlDataFactory::toStdout ( res );
		} else if ( algorithm.getValue ( ) == "exactMatchingAutomaton" ) {
			string::String pattern = alib::XmlDataFactory::fromTokens < string::String > ( std::move ( sax::FromXMLParserHelper::parseInput(true, patternInput).front ( ) ) );

			measurements::end ( );
			measurements::start ( "Algorithm", measurements::Type::MAIN );

			automaton::Automaton automaton = stringology::exact::ExactMatchingAutomaton::construct ( pattern );

			measurements::end ( );
			measurements::start ( "Output write", measurements::Type::AUXILIARY );

			alib::XmlDataFactory::toStdout ( automaton );
		} else if ( algorithm.getValue ( ) == "exactFactorAutomaton" ) {
			string::String subject = alib::XmlDataFactory::fromTokens < string::String > ( std::move ( sax::FromXMLParserHelper::parseInput(true, subjectInput).front ( ) ) );

			measurements::end ( );
			measurements::start ( "Algorithm", measurements::Type::MAIN );

			automaton::Automaton automaton = stringology::exact::ExactFactorAutomaton::construct ( subject );

			measurements::end ( );
			measurements::start ( "Output write", measurements::Type::AUXILIARY );

			alib::XmlDataFactory::toStdout ( automaton );
		} else if ( algorithm.getValue ( ) == "exactSubsequenceAutomaton" ) {
			string::String subject = alib::XmlDataFactory::fromTokens < string::String > ( std::move ( sax::FromXMLParserHelper::parseInput(true, subjectInput).front ( ) ) );

			measurements::end ( );
			measurements::start ( "Algorithm", measurements::Type::MAIN );

			automaton::Automaton automaton = stringology::exact::ExactSubsequenceAutomaton::construct ( subject );

			measurements::end ( );
			measurements::start ( "Output write", measurements::Type::AUXILIARY );

			alib::XmlDataFactory::toStdout ( automaton );
		} else if ( algorithm.getValue ( ) == "exactNondeterministicSubsequenceAutomaton" ) {
			string::String subject = alib::XmlDataFactory::fromTokens < string::String > ( std::move ( sax::FromXMLParserHelper::parseInput(true, subjectInput).front ( ) ) );

			measurements::end ( );
			measurements::start ( "Algorithm", measurements::Type::MAIN );

			automaton::Automaton automaton = stringology::exact::ExactNondeterministicSubsequenceAutomaton::construct ( subject );

			measurements::end ( );
			measurements::start ( "Output write", measurements::Type::AUXILIARY );

			alib::XmlDataFactory::toStdout ( automaton );
		} else if ( algorithm.getValue ( ) == "exactMultiNondeterministicSubsequenceAutomaton" ) {
			std::set < string::String > subjects;

			for ( std::deque < sax::Token > & tokens : sax::FromXMLParserHelper::parseInput(false, subjectInput) )
				subjects.insert ( alib::XmlDataFactory::fromTokens < string::String > ( std::move ( tokens ) ) );

			measurements::end ( );
			measurements::start ( "Algorithm", measurements::Type::MAIN );

			automaton::Automaton automaton = stringology::exact::ExactMultiNondeterministicSubsequenceAutomaton::construct ( subjects );

			measurements::end ( );
			measurements::start ( "Output write", measurements::Type::AUXILIARY );

			alib::XmlDataFactory::toStdout ( automaton );
		} else if ( algorithm.getValue ( ) == "factorOracleAutomaton" ) {
			string::String pattern = alib::XmlDataFactory::fromTokens < string::String > ( std::move ( sax::FromXMLParserHelper::parseInput(true, patternInput).front ( ) ) );

			measurements::end ( );
			measurements::start ( "Algorithm", measurements::Type::MAIN );

			automaton::Automaton automaton = stringology::exact::FactorOracleAutomaton::construct ( pattern );

			measurements::end ( );
			measurements::start ( "Output write", measurements::Type::AUXILIARY );

			alib::XmlDataFactory::toStdout ( automaton );
		} else if ( algorithm.getValue ( ) == "suffixAutomaton" ) {
			string::String pattern = alib::XmlDataFactory::fromTokens < string::String > ( std::move ( sax::FromXMLParserHelper::parseInput(true, patternInput).front ( ) ) );

			measurements::end ( );
			measurements::start ( "Algorithm", measurements::Type::MAIN );

			automaton::Automaton automaton = stringology::exact::SuffixAutomaton::construct ( pattern );

			measurements::end ( );
			measurements::start ( "Output write", measurements::Type::AUXILIARY );

			alib::XmlDataFactory::toStdout ( automaton );
		} else if ( algorithm.getValue ( ) == "borderArray" ) {
			string::String subject = alib::XmlDataFactory::fromTokens < string::String > ( std::move ( sax::FromXMLParserHelper::parseInput(true, subjectInput).front ( ) ) );

			measurements::end ( );
			measurements::start ( "Algorithm", measurements::Type::MAIN );

			std::vector < unsigned > borderArray = stringology::exact::BorderArray::construct ( subject );

			measurements::end ( );
			measurements::start ( "Output write", measurements::Type::AUXILIARY );

			alib::XmlDataFactory::toStdout ( borderArray );
		} else if ( algorithm.getValue ( ) == "suffixTrie" ) {
			string::String subject = alib::XmlDataFactory::fromTokens < string::String > ( std::move ( sax::FromXMLParserHelper::parseInput(true, subjectInput).front ( ) ) );

			measurements::end ( );
			measurements::start ( "Algorithm", measurements::Type::MAIN );

			indexes::SuffixTrieFinalMark suffixTrie = stringology::indexing::SuffixTrie::construct ( subject );

			measurements::end ( );
			measurements::start ( "Output write", measurements::Type::AUXILIARY );

			alib::XmlDataFactory::toStdout ( suffixTrie );
		} else {
			throw exception::CommonException ( "Invalid algorithm" );
		}

		measurements::end ( );
		measurements::end ( );

		if ( measure.getValue ( ) ) std::cmeasure << measurements::results ( ) << std::endl;

		return 0;
	} catch ( const exception::CommonException & exception ) {
		alib::XmlDataFactory::toStdout ( exception );
		return 1;
	} catch ( const TCLAP::ArgException & exception ) {
		std::cout << exception.error ( ) << std::endl;
		return 2;
	} catch ( const std::exception & exception ) {
		std::cerr << "Exception caught: " << exception.what ( ) << std::endl;
		return 3;
	} catch ( ... ) {
		std::cerr << "Unknown exception caught." << std::endl;
		return 127;
	}
}
